#pragma once

#include "esphome/components/select/select.h"
#include "../seeed_mr24hpc1.h"

namespace esphome::seeed_mr24hpc1 {

class SceneModeSelect final : public select::Select, public Parented<MR24HPC1Component> {
 public:
  SceneModeSelect() = default;

 protected:
  void control(size_t index) override;
};

}  // namespace esphome::seeed_mr24hpc1
