#pragma once

#include "esphome/components/es8388/es8388.h"
#include "esphome/components/select/select.h"

namespace esphome::es8388 {

class DacOutputSelect final : public select::Select, public Parented<ES8388> {
 protected:
  void control(size_t index) override;
};

}  // namespace esphome::es8388
