import esphome.config_validation as cv

CONFIG_SCHEMA = cv.invalid(
    "The esp32_hall component has been removed as of ESPHome 2025.7.0. See https://github.com/esphome/esphome/pull/9117 for details."
)
