import esphome.codegen as cg

st7789v_ns = cg.esphome_ns.namespace("st7789v")

DEPRECATED_COMPONENT = """
The 'st7789v' component is deprecated and no new functionality will be added to it.
PRs should target the newer and more performant 'mipi_spi' component.
"""
