CODEOWNERS = ["@clydebarrow"]

CONF_DRAW_FROM_ORIGIN = "draw_from_origin"

DEPRECATED_COMPONENT = """
The 'qspi_dbi' component is deprecated and no new models will be added to it.
New model PRs should target the newer and more performant 'mipi_spi' component.
"""
