#pragma once

// This file is not used by the runtime, instead, a version is generated during
// compilation with the actual build info values.
//
// This file is only used by static analyzers and IDEs.

#define ESPHOME_CONFIG_HASH 0x12345678U  // NOLINT
#define ESPHOME_BUILD_TIME 1700000000    // NOLINT
#define ESPHOME_COMMENT_SIZE 1           // NOLINT
static const char ESPHOME_BUILD_TIME_STR[] = "2024-01-01 00:00:00 +0000";
static const char ESPHOME_COMMENT_STR[] = "";
