#pragma once

// HaierProtocol
#include <utils/haier_log.h>

namespace esphome::haier {

// This file is called in the code generated by python script
// Do not use it directly!
void init_haier_protocol_logging();

}  // namespace esphome::haier
