import esphome.codegen as cg

st7735_ns = cg.esphome_ns.namespace("st7735")

DEPRECATED_COMPONENT = """
The 'st7735' component is deprecated and no new models will be added to it.
New model PRs should target the newer and more performant 'mipi_spi' component.
"""
